//------------------------------------------------------------------
#property copyright "mladen"
#property link      "mladen"
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  Orange
#property indicator_color3  Orange
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_minimum 0
#property indicator_maximum 100
#property indicator_level1  70
#property indicator_level2  30

extern string TimeFrame   = "Current time frame";
extern double Length      = 14;
extern int    Price       = PRICE_CLOSE;
extern double SmoothSpeed =  1;

double rsi[];
double rsida[];
double rsidb[];
double slope[];
string indicatorFileName;
bool   returnBars;
int    timeFrame;

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,rsi);
   SetIndexBuffer(1,rsida);
   SetIndexBuffer(2,rsidb);
   SetIndexBuffer(3,slope);

      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame == "returnBars";     if (returnBars)     return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);

   IndicatorShortName("Rsi of smoother ("+DoubleToStr(Length,2)+")");
   return(0);
}
int deinit() { return(0); }

//
//
//
//
//

int start()
{
   int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { rsi[0] = limit+1; return(0); }
         if (timeFrame!=Period())
         {
            limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
            if (slope[limit]==-1) CleanPoint(limit,rsida,rsidb);
            for (i=limit; i>=0; i--)
            {
               int y = iBarShift(NULL,timeFrame,Time[i]);               
                  slope[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Length,Price,SmoothSpeed,3,y);
                  rsi[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Length,Price,SmoothSpeed,0,y);
                  rsida[i] = EMPTY_VALUE;
                  rsidb[i] = EMPTY_VALUE;
                     if (slope[i] == -1) PlotPoint(i,rsida,rsidb,rsi);
            }
            return(0);                  
         }

   //
   //
   //
   //
   //
   
   if (slope[limit]==-1) CleanPoint(limit,rsida,rsidb);
   for(i=limit; i>=0; i--)
   {
      double mom  = iMA(NULL,0,1,0,MODE_SMA,Price,i)-iMA(NULL,0,1,0,MODE_SMA,Price,i+1);
      double moa  = MathAbs(mom);
      double tmom = iSmooth(mom,Length,SmoothSpeed,i,0);
      double tmoa = iSmooth(moa,Length,SmoothSpeed,i,1);
      if (tmoa != 0)
           rsi[i] = MathMax(MathMin((tmom/tmoa+1.0)*50.0,100.00),0.00); 
      else rsi[i] = 50;
      slope[i] = slope[i+1];
         if (rsi[i]>rsi[i+1]) slope[i] =  1;
         if (rsi[i]<rsi[i+1]) slope[i] = -1;
                     rsida[i] = EMPTY_VALUE;
                     rsidb[i] = EMPTY_VALUE;
                     if (slope[i] == -1) PlotPoint(i,rsida,rsidb,rsi);
   }
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workSmooth[][10];
double iSmooth(double price,int length, double speed, int r, int instanceNo=0)
{
   if (ArrayRange(workSmooth,0)!=Bars) ArrayResize(workSmooth,Bars); instanceNo *= 5; r = Bars-r-1;
 	if(r<=2) { workSmooth[r][instanceNo] = price; workSmooth[r][instanceNo+2] = price; workSmooth[r][instanceNo+4] = price; return(price); }
   
   //
   //
   //
   //
   //

	double beta  = 0.45*(length-1.0)/(0.45*(length-1.0)+2.0);
	double alpha = MathPow(beta,speed);
   	  workSmooth[r][instanceNo+0] =  price+alpha*(workSmooth[r-1][instanceNo]-price);
	     workSmooth[r][instanceNo+1] = (price - workSmooth[r][instanceNo])*(1-alpha)+alpha*workSmooth[r-1][instanceNo+1];
	     workSmooth[r][instanceNo+2] =  workSmooth[r][instanceNo+0] + workSmooth[r][instanceNo+1];
	     workSmooth[r][instanceNo+3] = (workSmooth[r][instanceNo+2] - workSmooth[r-1][instanceNo+4])*MathPow(1.0-alpha,2) + MathPow(alpha,2)*workSmooth[r-1][instanceNo+3];
	     workSmooth[r][instanceNo+4] =  workSmooth[r][instanceNo+3] + workSmooth[r-1][instanceNo+4]; 
   return(workSmooth[r][instanceNo+4]);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}
void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
         {
            if (first[i+2] == EMPTY_VALUE) 
                 { first[i]  = from[i]; first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
            else { second[i] = from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
         }
   else  { first[i] = from[i]; second[i] = EMPTY_VALUE; }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}